unit DBFserver;

interface

uses SysUtils;

Const
  MaxMemoSize=100;  { set to max memo size you will need }
  MaxTags=15;
  { Constants used by sx_Use() }
  sx_DBFCDX=2;
  sx_DBFNTX=1;
  sx_DBFNSX=3;
  sx_READWRITE=0;
  sx_EXCLUSIVE=2;
  { Constants used by sx_Replace() }
  r_integer=1;
  r_long=2;
  r_double=8;
  r_julian=32;
  r_logical=128;
  r_char=1024;
  r_datestr=1056;
  r_memo=3072;
  r_bitmap=4096;
  r_bloblfile=8192;

Type
  DBServer=class(Tobject)
    { Field get/replace functions}
    { Get field info, pass in field name -  Return Value of 'fldname' }
    function  s(fldname:string): string;    { full string, no trim() }
    function  st(fldname:string):string;    { trim()'ed value }
    function  l(fldname:string): longint;   { field info as Longint }
    function  i(fldname:string): integer;   { field info as Integer }
    function  b(fldname:string): boolean;   { field info as Boolean }
    function  f(fldname:string): double;    { field info as Float }
    function  m(fldname:string): pchar;     { field info as Memo }
    function  d(fldname:string): string;    { as date: 04/15/95 style }
    function  dj(fldname:string):longint;   { as Julian date }
    {  Replace field with "newval", pass in field name
       and var of appropriate data type as needed - Replace 'fldname' with }
    procedure sa(fldname:string;newval:string);    { String }
    procedure la(fldname:string;newval:longint);   { Longint }
    procedure ia(fldname:string;newval:integer);   { Integer }
    procedure ba(fldname:string;newval:boolean);   { Boolean }
    procedure fa(fldname:string;newval:double);    { Double }
    procedure ma(fldname:string;newval:pchar);     { Memo (Pchar) }
    procedure da(fldname:string;newval:string);    { Date like '01/15/95' }
    { Standard Database commands }
    function  Alias:string;
    procedure Append;
    procedure Close;
    procedure CloseAll;
    procedure Delete;  { mark record as deleted }
    function  Deleted: boolean; { status of deletion flag for record }
    function  Eof: boolean;
    procedure Go(RecNo:longint);  { GoTo is reserved word, not used }
    procedure GoBottom;
    procedure GoTop;
    function  Lastrec: longint;
    procedure LoadTags;
    function  Lock: boolean;   { try lock until succeeds }
    function  aLock: boolean;  { try a few times, then return if fails }
    procedure Pack;
    function  RecCount: longint;
    function  RecNo: longint;
    function  Recall: boolean;  { unmark record as deleted }
    function  Seek(apattern:string): boolean;
    procedure Select(AliasName:string);
    procedure SetOrder(ToIndex:integer);  { by tag number }
    procedure Skip;
    procedure Skip2(ByCnt:integer);
    procedure TagOrder(OrderByTag:String);  { by tag name }
    procedure unLock;
    function  Use(dbfname:string): boolean;
    function  UseWithAlias(dbfname:string;AliasName:string): boolean;
    function  UseAliasExcl(dbfname:string;AliasName:string): boolean;
    function  UseExclusive(dbfname:string): boolean;
    procedure Zap;
    { Extended Database Commands }
    { Change to different alias and then back to original in-line }
    procedure xAppend(UseAlias:String);
    procedure xClose(UseAlias:String);
    procedure xDelete(UseAlias:String);
    function  xDeleted(UseAlias:String): boolean;
    function  xEof(UseAlias:String): boolean;
    procedure xGo(UseAlias:String;RecNo:longint);
    procedure xGoBottom(UseAlias:String);
    procedure xGoTop(UseAlias:String);
    function  xLastrec(UseAlias:String): longint;
    function  xLock(UseAlias:String): boolean;
    function  xaLock(UseAlias:String): boolean;
    function  xRecCount(UseAlias:String): longint;
    function  xRecNo(UseAlias:String): longint;
    function  xRecall(UseAlias:String): boolean;
    function  xSeek(UseAlias:String;apattern:string): boolean;
    procedure xSetOrder(UseAlias:String;ToIndex:integer);
    procedure xSkip(UseAlias:String);
    procedure xSkip2(UseAlias:String;ByCnt:integer);
    procedure xTagOrder(UseAlias:String;OrderByTag:String);
    procedure xunLock(UseAlias:String);

  public
    TagCnt: integer;
    Tags: array [1..MaxTags] of string[15];
    Keys: array [1..MaxTags] of string[82];
end;

var StrNull1:Pchar;  { global so it doesn't have to appear in every proc }
    StrNull2:Pchar;
    StrMemo:Pchar;
    DB:DBserver;  { used for all DBF accessing }

procedure dbShutDown; { Release DBserver resources }
function YN(aBool:Boolean):String; {Convert Boolean to string, 'Y','N' }
function ValidDate(DateAsString:String):Boolean;  { date valid? }
{ A few xBase classics }
function Empty(aStr:String):Boolean; { date strings and regular strings }
function Trim(aStr:String):String;   { trim off trailing spaces }
function lTrim(aStr:String):String;   { trim off leading spaces }
function PadC(aStr:String;InWidth:Integer):String; { pad center in width }
function PadL(aStr:String;InWidth:Integer):String; { right justify in width }
function PadR(aStr:String;InWidth:Integer):String; { left justify in width }
function Space(EmptySize:Integer):String;  { return string of spaces }
{ same as copy() }
function SubStr(aStr:String;Start:Integer;Count:Integer):String;

implementation

 var TempAlias:string[20];  { keep track of alias during p????? methods }

 { the following were translated from /rocket/sixcpp/stdafx.h }
  procedure sx_AppendBlank; far; external 'ROCKET';
  function  sx_Alias(AreaNo:Integer):Pchar; far; external 'ROCKET';
  function  sx_Bof:Integer; far; external 'ROCKET';
  procedure sx_Close; far; external 'ROCKET';
  procedure sx_CloseAll; far; external 'ROCKET';
  procedure sx_Commit; far; external 'ROCKET';
  procedure sx_Delete; far; external 'ROCKET';
  function  sx_Deleted:Integer; far; external 'ROCKET';
  function  sx_Eof:Integer; far; external 'ROCKET';
  function  sx_EvalNumeric(Expr:Pchar):Double; far; external 'ROCKET';
  function  sx_Found:Integer; far; external 'ROCKET';
  function  sx_GetDateJulian(FLDname:Pchar):Longint; far; external 'ROCKET';
  function  sx_GetDateString(FLDname:Pchar):Pchar; far; external 'ROCKET';
  function  sx_GetDouble(FLDname:Pchar):Double; far; external 'ROCKET';
  function  sx_GetInteger(FLDname:Pchar):Integer; far; external 'ROCKET';
  function  sx_GetLogical(FLDname:Pchar):Integer; far; external 'ROCKET';
  function  sx_GetLong(FLDname:Pchar):Longint; far; external 'ROCKET';
  function  sx_GetMemo(FLDname:Pchar;LineWidth:Integer):Pchar;
    far; external 'ROCKET';
  function  sx_GetString(FLDname:Pchar):Pchar; far; external 'ROCKET';
  function  sx_GetTrimString(FLDname:Pchar):Pchar; far; external 'ROCKET';
  procedure sx_Go(ToRec:LongInt); far; external 'ROCKET';
  procedure sx_GoBottom; far; external 'ROCKET';
  procedure sx_GoTop; far; external 'ROCKET';
  function  sx_IndexOrd:Integer; far; external 'ROCKET';
  function  sx_IndexKey:Pchar; far; external 'ROCKET';
  function  sx_Locked(RecNo:LongInt):Integer; far; external 'ROCKET';
  procedure sx_MemDealloc(aPointer:Pchar); far; external 'ROCKET';
  procedure sx_Pack; far; external 'ROCKET';
  procedure sx_Recall; far; external 'ROCKET';
  function  sx_RecCount:Longint; far; external 'ROCKET';
  function  sx_RecNo:Longint; far; external 'ROCKET';
  procedure sx_Replace(FLDname:Pchar;FldType:Integer;PtrData:Pchar);
    far; external 'ROCKET';
  function  sx_Rlock(RecNo:Longint):Integer; far; external 'ROCKET';
  function  sx_Seek(aPattern:Pchar):Integer; far; external 'ROCKET';
  function  sx_Select(AreaNo:Integer):Integer; far; external 'ROCKET';
  procedure sx_SetDeleted(OnOff:Integer); far; external 'ROCKET';
  procedure sx_SetExact(OnOff:Integer); far; external 'ROCKET';
  function  sx_SetHandles(ToCnt:Integer):Integer; far; external 'ROCKET';
  function  sx_SetOrder(ToIndexNo:Integer):Integer; far; external 'ROCKET';
  procedure sx_SetStringType(cstyle:Integer); far; external 'ROCKET';
  procedure sx_Skip(MoveCnt:Longint); far; external 'ROCKET';
  function  sx_TagArea(TagName:Pchar):Integer; far; external 'ROCKET';
  function  sx_TagName(TagIndex:Integer):Pchar; far; external 'ROCKET';
  procedure sx_Unlock(RecNo:Longint); far; external 'ROCKET';
  function  sx_Use(Fname:pchar;dAlias:pchar;OpenMode:Integer;
    RDDtype:Integer): Integer;
    far; external 'ROCKET';
  function  sx_WorkArea(AliasName:Pchar):Integer; far; external 'ROCKET';
  procedure sx_Zap; far; external 'ROCKET';

  function Space(EmptySize:Integer):String;  { return string of spaces }
  var tt:string;
  begin
    tt:='                                                 ';
    Result:=copy(tt+tt+tt+tt+tt,1,EmptySize);
  end;

  function PadC(aStr:String;InWidth:Integer):String; { pad center in width }
  var ii,ll:integer;
  begin
    ll:=length(aStr);
    if ll>InWidth then Result:=copy(aStr,1,Inwidth)  { truncate }
    else
    begin
      ii:=(InWidth-ll) div 2;
      if ii>0 then Result:=space(ii)+aStr;
      ll:=length(Result);
      if ll<InWidth then Result:=Result+space(InWidth-ll)
    end;
  end;

  function PadL(aStr:String;InWidth:Integer):String; { right justify in width }
  var ll:integer;
  begin
    ll:=length(aStr);
    if ll>InWidth then Result:=copy(aStr,1,Inwidth)  { truncate }
    else Result:=space(InWidth-ll)+aStr;
  end;

  function PadR(aStr:String;InWidth:Integer):String; { left justify in width }
  var ll:integer;
  begin
    ll:=length(aStr);
    if ll>InWidth then Result:=copy(aStr,1,Inwidth)  { truncate }
    else Result:=aStr+space(InWidth-ll);
  end;

  function YN(aBool:Boolean):String;
  begin
    Result:='N';
    if aBool then Result:='Y';
  end;

  function ValidDate(DateAsString:String):Boolean;  { date valid? }
  var tdate:TDateTime;
  begin
    try
      tdate:=StrToDate(DateAsString);
      Result:=True;  { if we made it here, it was OK }
    except
      { Must turn-off option on Environment Options window
        "Break on Exception" in "Debugging" section while testing }
      Result:=False;
    end;
  end;

  function Trim(aStr:String):String;   { trim off trailing spaces }
  var ii,kk,ll:integer;
  begin
    ll:=length(aStr);
    Result:=aStr;
    if ll>0 then
    begin
      kk:=0;
      for ii:=ll downto 1 do
      begin
        if aStr[ii]<>#32 then
        begin
          kk:=ii;
          break;
        end;
      end;
      if kk>0 then Result:=copy(astr,1,kk)
      else Result:='';
    end;
  end;

  function SubStr(aStr:String;Start:Integer;Count:Integer):String;   { same as copy() }
  begin
    Result:=Copy(aStr,Start,Count);
  end;

  function lTrim(aStr:String):String;   { trim off trailing spaces }
  var ii,kk,ll:integer;
  begin
    ll:=length(aStr);
    Result:=aStr;
    if ll>0 then
    begin
      kk:=0;
      for ii:=1 to ll do
      begin
        if aStr[ii]<>#32 then
        begin
          kk:=ii;
          break;
        end;
      end;
      if kk>0 then Result:=copy(astr,kk,254)
      else Result:='';
    end;
  end;

  function Empty(aStr:String):Boolean;
  var ii,ll:integer;
      res:boolean;
  begin
    if length(aStr)=0 then res:=true
    else
    begin
      ll:=length(aStr);
      if (ll=8) or (ll=10) then { check for date? }
      begin
        if (aStr[3]=#47) and (aStr[6]=#47) then { 3 and 6 chars / }
        begin
          ll:=2; { only need to test first 2 chars of dates }
        end;
      end;
      res:=True;
      for ii:=1 to ll do
      begin
        if aStr[ii]<>#32 then
        begin
          res:=False;
          break;
        end;
      end;
    end;
    Result:=res;
  end;

  procedure PushAlias(ToAlias:String);
  begin
    TempAlias:=db.Alias;
    if not empty(ToAlias) then db.Select(ToAlias);
  end;

  procedure PopAlias;
  begin
    if not Empty(TempAlias) then db.Select(TempAlias);
  end;

  { True/False tester for DLL boolean (integer) return values }
  function tf(AnInt:Integer):Boolean;
  begin
    Result:=AnInt<>0; { True=Any Non-Zero Value }
  end;

  procedure dbShutDown;
  begin
    db.free;
    StrDispose(StrNull1);
    StrDispose(StrNull2);
    StrDispose(StrMemo);
  end;

  function CoreFile(FullPath:string):string;
  var ii:integer;
  { Get core file name for aliases, no path, no extension }
  begin
    ii:=pos('\',FullPath);
    while ii>0 do
    begin
      FullPath:=Copy(FullPath,ii+1,100);
      ii:=pos('\',FullPath);
    end;
    ii:=pos('.',FullPath);
    if ii>1 then FullPath:=Copy(FullPath,1,ii-1);
    Result:=FullPath;
  end;

  procedure delay(ForSeconds:integer);
  { delay for interval in seconds }
  var tt:TDateTime;
      hr,thr,mn,sc,ms:word;
      ll,cur,rng:LongInt;
  begin
    tt:=now;
    rng:=ForSeconds;
    DecodeTime(tt,hr,mn,sc,ms);
    thr:=hr;
    cur:=(hr*3600)+(mn*60)+sc;
    ll:=(hr*3600)+(mn*60)+sc;
    while rng>(ll-cur) do
    begin
      tt:=now;
      DecodeTime(tt,hr,mn,sc,ms);
      if hr<thr then hr:=hr+24;  { anyone work at midnight? }
      ll:=(hr*3600)+(mn*60)+sc;
    end;
  end;

  function DBServer.s(fldname:string): string;
  begin
    Result:=StrPas(sx_GetString(StrPCopy(StrNull1,fldname)));
  end;

  function DBServer.st(fldname:string): string;
  begin
    Result:=StrPas(sx_GetTrimString(StrPCopy(StrNull1,fldname)));
  end;

  function DBServer.l(fldname:string): longint;
  begin
    Result:=sx_GetLong(StrPCopy(StrNull1,fldname));
  end;

  function DBServer.i(fldname:string): integer;
  begin
    Result:=sx_GetInteger(StrPCopy(StrNull1,fldname));
  end;

  function DBServer.b(fldname:string): boolean;
  begin
    Result:=tf(sx_GetLogical(StrPCopy(StrNull1,fldname)));
  end;

  function DBServer.f(fldname:string): double;
  begin
    { Minor bug: can't use sx_GetDouble yet }
    { S/B Result:=sx_GetDouble(StrPCopy(StrNull1,fldname)); }
    Result:=StrToFloat(StrPas(sx_GetString(StrPCopy(StrNull1,fldname))));
  end;

  function DBServer.m(fldname:string):pchar;
  var tPchar:Pchar;
  begin
    tPchar:=sx_GetMemo(StrPCopy(StrNull1,fldname),0);
    StrCopy(StrMemo,tPchar);
    sx_MemDealloc(tPchar);
    Result:=StrMemo;
  end;

  function DBServer.d(fldname:string): string;  { date of form 04/15/95 }
  begin
    Result:=StrPas(sx_GetDateString(StrPCopy(StrNull1,fldname)));
  end;

  function DBServer.dj(fldname:string):longint; { date as Julian date}
  begin
    Result:=sx_GetDateJulian(StrPCopy(StrNull1,fldname));
  end;

  procedure DBServer.sa(fldname:string;newval:string);
  begin
    sx_Replace(StrPCopy(StrNull1,fldname),r_char,StrPCopy(StrNull2,newval));
  end;

  procedure DBServer.la(fldname:string;newval:longint);
  begin
    sx_Replace(StrPCopy(StrNull1,fldname),r_long,@newval);
  end;

  procedure DBServer.ia(fldname:string;newval:integer);
  begin
    sx_Replace(StrPCopy(StrNull1,fldname),r_integer,@newval);
  end;

  procedure DBServer.ba(fldname:string;newval:boolean);
  var ii:integer;
  begin
    if newval then ii:=1
              else ii:=0;
    sx_Replace(StrPCopy(StrNull1,fldname),r_logical,@ii);
  end;

  procedure DBServer.fa(fldname:string;newval:double);
  begin
    sx_Replace(StrPCopy(StrNull1,fldname),r_double,@newval);
  end;

  procedure DBServer.ma(fldname:string;newval:pchar);
  begin
    StrCopy(StrMemo,newval);
    sx_Replace(StrPCopy(StrNull1,fldname),r_memo,StrMemo);
  end;

  procedure DBServer.da(fldname:string;newval:string);
  begin
    { pass in string of form '01/15/95', invalid dates force field to blank }
    sx_Replace(StrPCopy(StrNull1,fldname),r_datestr,StrPCopy(StrNull2,newval));
  end;

  function DBServer.Alias:string;
  begin
    if sx_WorkArea(Nil)>0 then
      Result:=StrPas(sx_Alias(0))
    else Result:='';
  end;

  procedure DBServer.Append;
  begin
    sx_AppendBlank;
  end;

  procedure DBServer.GoBottom;
  begin
    sx_GoBottom;
  end;

  procedure DBServer.Close;
  begin
    sx_Close;
  end;

  procedure DBServer.CloseAll;
  begin
    sx_CloseAll;
  end;

  procedure DBServer.Delete;  { mark record as deleted }
  begin
    sx_Delete;
  end;

  function DBServer.Deleted: boolean; { status of deletion flag of record }
  begin
    Result:=tf(sx_Deleted);
  end;

  function DBServer.Eof: boolean;
  begin
    result:=tf(sx_Eof);
  end;

  procedure DBServer.Go(RecNo:longint);
  begin
    sx_Go(RecNo);
  end;

  function DBServer.LastRec: longint;
  begin
    Result:=sx_Reccount;
  end;

  function DBServer.Lock: boolean;   { try lock until succeeds }
  var ii:integer;
      res:boolean;
  begin
    res:=False;
    ii:=0;
    while (ii<100) and (not res) do  { bail-out after 300 seconds }
    begin
      res:=tf(sx_rLock(sx_Recno));
      if not res then delay(3);
      ii:=ii+1;
    end;
    Result:=res;
  end;

  function DBServer.aLock: boolean;  { try a few times then return }
  var ii:integer;
      res:boolean;
  begin
    ii:=0;
    res:=False;
    while (ii<3) and (not res) do  { timeout=3*3=9 seconds }
    begin
      res:=tf(sx_rLock(sx_Recno));
      if not res then delay(3);
      ii:=ii+1;
    end;
    Result:=res;
  end;

  procedure DBServer.LoadTags;
  var ii,CurIndex:integer;
  begin
    tagcnt:=0;
    CurIndex:=sx_IndexOrd;
    for ii:=1 to MaxTags do
    begin
      tags[ii]:=StrPas(sx_TagName(ii));
      if length(tags[ii])>0 then
      begin
        tagcnt:=tagcnt+1;
        sx_SetOrder(ii);
        keys[ii]:=StrPas(sx_IndexKey);
      end else break;
    end;
    sx_SetOrder(CurIndex);
  end;

  procedure DBServer.Pack;
  begin
    sx_Pack;
  end;

  function DBServer.Recall: boolean;  { unmark record as deleted }
  begin
    sx_Recall;
  end;

  function DBServer.RecCount: longint;
  begin
    Result:=sx_RecCount;
  end;

  function DBServer.RecNo: longint;
  begin
    Result:=sx_RecNo;
  end;

  function DBServer.Seek(apattern:string): boolean;
  begin
    Result:=tf(sx_Seek(StrPCopy(StrNull1,apattern)));
  end;

  procedure DBServer.Select(AliasName:string);
  begin
    sx_Select(sx_WorkArea(StrPCopy(StrNull1,AliasName)));
  end;

  procedure DBServer.SetOrder(ToIndex:integer);
  begin
    sx_SetOrder(ToIndex);
  end;

  procedure DBServer.Skip;
  begin
    sx_Skip(1);
  end;

  procedure DBServer.Skip2(ByCnt:integer);
  begin
    sx_Skip(ByCnt);
  end;

  procedure DBServer.TagOrder(OrderByTag:String);
  begin
    sx_SetOrder(sx_TagArea(StrPCopy(StrNull1,OrderByTag)));
  end;

  procedure DBServer.GoTop;
  begin
    sx_GoTop;
  end;

  procedure DBServer.unLock;
  begin
    sx_Commit;
    sx_unLock(sx_Recno);
  end;

  function DBServer.Use(dbfname:string): boolean;
  var aliasname:string;
      ii:integer;
  begin
    aliasname:=CoreFile(dbfname);
    ii:=sx_Use(StrPCopy(StrNull1,dbfname),
      StrPCopy(StrNull2,aliasname),sx_READWRITE,sx_DBFCDX);
    if ii>0 then
    begin
      sx_SetOrder(1);
      Result:=True;
    end else Result:=False;
  end;

  function DBServer.UseWithAlias(dbfname:string;AliasName:string): boolean;
  var ii:integer;
  begin
    ii:=sx_Use(StrPCopy(StrNull1,dbfname),
      StrPCopy(StrNull2,aliasname),sx_READWRITE,sx_DBFCDX);
    if ii>0 then
    begin
      sx_SetOrder(1);
      Result:=True;
    end else Result:=False;
  end;

  function DBServer.UseAliasExcl(dbfname:string;AliasName:string): boolean;
  var ii:integer;
  begin
    ii:=sx_Use(StrPCopy(StrNull1,dbfname),
      StrPCopy(StrNull2,aliasname),sx_EXCLUSIVE,sx_DBFCDX);
    if ii>0 then
    begin
      sx_SetOrder(1);
      Result:=True;
    end else Result:=False;
  end;

  function DBServer.UseExclusive(dbfname:string): boolean;
  var aliasname:string;
      ii:integer;
  begin
    aliasname:=CoreFile(dbfname);
    ii:=sx_Use(StrPCopy(StrNull1,dbfname),
      StrPCopy(StrNull2,aliasname),sx_EXCLUSIVE,sx_DBFCDX);
    if ii>0 then
    begin
      sx_SetOrder(1);
      Result:=True;
    end else Result:=False;
  end;

  procedure DBServer.Zap;
  begin
    sx_Zap;
  end;

  { Begin Extended Database Command Definitions }

  procedure DBServer.xAppend(UseAlias:String);
  begin
    PushAlias(UseAlias);
    self.Append;
    popalias;
  end;

  procedure DBServer.xGoBottom(UseAlias:String);
  begin
    PushAlias(UseAlias);
    self.GoBottom;
    popalias;
  end;

  procedure DBServer.xClose(UseAlias:String);
  begin
    PushAlias(UseAlias);
    self.Close;
    popalias;
  end;

  procedure DBServer.xDelete(UseAlias:String);
  begin
    PushAlias(UseAlias);
    self.Delete;
    popalias;
  end;

  function  DBServer.xDeleted(UseAlias:String): boolean;
  begin
    PushAlias(UseAlias);
    Result:=self.Deleted;
    popalias;
  end;

  function  DBServer.xEof(UseAlias:String): boolean;
  begin
    PushAlias(UseAlias);
    Result:=self.Eof;
    popalias;
  end;

  procedure DBServer.xGo(UseAlias:String;RecNo:longint);
  begin
    PushAlias(UseAlias);
    self.Go(RecNo);
    popalias;
  end;

  function  DBServer.xLastrec(UseAlias:String): longint;
  begin
    PushAlias(UseAlias);
    Result:=self.LastRec;
    popalias;
  end;

  function  DBServer.xLock(UseAlias:String): boolean;
  begin
    PushAlias(UseAlias);
    Result:=self.Lock;
    popalias;
  end;

  function  DBServer.xaLock(UseAlias:String): boolean;
  begin
    PushAlias(UseAlias);
    Result:=self.aLock;
    popalias;
  end;

  function  DBServer.xRecCount(UseAlias:String): longint;
  begin
    PushAlias(UseAlias);
    Result:=self.RecCount;
    popalias;
  end;

  function  DBServer.xRecNo(UseAlias:String): longint;
  begin
    PushAlias(UseAlias);
    Result:=self.RecNo;
    popalias;
  end;

  function  DBServer.xRecall(UseAlias:String): boolean;
  begin
    PushAlias(UseAlias);
    Result:=self.Recall;
    popalias;
  end;

  function  DBServer.xSeek(UseAlias:String;apattern:string): boolean;
  begin
    PushAlias(UseAlias);
    Result:=self.Seek(apattern);
    popalias;
  end;

  procedure DBServer.xSetOrder(UseAlias:String;ToIndex:integer);
  begin
    PushAlias(UseAlias);
    self.SetOrder(ToIndex);
    popalias;
  end;

  procedure DBServer.xSkip2(UseAlias:String;ByCnt:integer);
  begin
    PushAlias(UseAlias);
    self.Skip2(ByCnt);
    popalias;
  end;

  procedure DBServer.xSkip(UseAlias:String);
  begin
    PushAlias(UseAlias);
    self.Skip;
    popalias;
  end;

  procedure DBServer.xTagOrder(UseAlias:String;OrderByTag:String);
  begin
    PushAlias(UseAlias);
    self.TagOrder(OrderByTag);
    popalias;
  end;

  procedure DBServer.xGoTop(UseAlias:String);
  begin
    PushAlias(UseAlias);
    self.GoTop;
    popalias;
  end;

  procedure DBServer.xunLock(UseAlias:String);
  begin
    PushAlias(UseAlias);
    self.unLock;
    popalias;
  end;


initialization

  sx_SetStringType(1);
  sx_SetHandles(40);
  sx_SetDeleted(0);   { show records marked as deleted }
  db:=dbserver.create;
  StrNull1:=StrAlloc(80);
  StrNull2:=StrAlloc(80);
  StrMemo:=StrAlloc(MaxMemoSize);

end.

