unit DBFserver;

interface

uses SysUtils;

Const
  maxtags=15;
  sx_DBFCDX=2;
  sx_DBFNTX=1;
  sx_DBFNSX=3;
  sx_READWRITE=0;
  sx_EXCLUSIVE=2;
  r_integer=1;
  r_long=2;
  r_double=8;
  r_julian=32;
  r_logical=128;
  r_char=1024;
  r_datestr=1056;

Type
  DBServer=class(Tobject)
    { Field get/assign functions}
    { Get field info, pass in field name }
    function  s(fldname:string): string;  { full string, no trim() }
    function  st(fldname:string):string;  { trim()'ed value }
    function  l(fldname:string): longint;
    function  i(fldname:string): integer;
    function  b(fldname:string): boolean;
    function  f(fldname:string): double;  { float }
    function  m(fldname:string): pchar;   { memo }
    function  d(fldname:string): string;  { returns: 04/15/95 style }
    function  dj(fldname:string):longint; { date as Julian date}
    { Assign "newval" to field, pass in field name and var of data type as needed }
    procedure sa(fldname:string;newval:string);
    procedure la(fldname:string;newval:longint);
    procedure ia(fldname:string;newval:integer);
    procedure ba(fldname:string;newval:boolean);
    procedure fa(fldname:string;newval:double);
    procedure ma(fldname:string;newval:pchar);
    procedure da(fldname:string;newval:string);
    { Database commands }
    function  Alias:string;
    procedure Append;
    procedure Bottom;
    procedure Close;
    procedure CloseAll;
    procedure Delete;  { mark record as deleted }
    function  Deleted: boolean; { status of deletion flag for record }
    function  Eof: boolean;
    procedure Go(recnum:longint);
    function  Lastrec: longint;
    procedure LoadTags;
    function  Lock: boolean;   { try lock until succeeds }
    function  aLock: boolean;  { try a few times, then return if fails }
    procedure Pack;
    function  RecCount: longint;
    function  RecNo: longint;
    function  Recall: boolean;  { unmark record as deleted }
    function  Seek(apattern:string): boolean;
    procedure Select(AliasName:string);
    procedure SetOrder(ToIndex:integer);  { by tag number }
    procedure Skip2(ByCnt:integer);
    procedure Skip;
    procedure TagOrder(OrderByTag:String);  { by tag name }
    procedure Top;
    procedure unLock;
    function  Use(dbfname:string): boolean;
    function  UseWithAlias(dbfname:string;AliasName:string): boolean;
    function  UseAliasExcl(dbfname:string;AliasName:string): boolean;
    function  UseExclusive(dbfname:string): boolean;
    procedure Zap;
  public
    TagCnt: integer;
    Tags: array [1..MaxTags] of string[15];
    Keys: array [1..MaxTags] of string[82];
  end;

var StrNull1:Pchar;  { global so it doesn't have to appear in every proc }
    StrNull2:Pchar;
    DB:DBserver;  { used for all DBF accessing }

    { StrNull1,StrNull2 are used to pass in strings to DLL routines
      And should be cleared in the main Begin..End of the Project
      module (PROJECT.DPR file), they are allocated in the Initialization
      Section below of this Unit.
      Create the main "db" variable instance, as follows:

      Project1.dpr file:

        begin
          dbf:=DBserver.create;
          Application.CreateForm(TForm1, Form1);
          Application.Run;
          dbf.free;
          StrDispose(StrNull1);
          StrDispose(StrNull2);
        end.

      }


implementation

 { the following were adapted from /rocket/sixcpp/stdafx.h }
  procedure sx_AppendBlank; far; external 'ROCKET';
  function  sx_Alias(AreaNo:Integer):Pchar; far; external 'ROCKET';
  function  sx_Bof:Integer; far; external 'ROCKET';
  procedure sx_Close; far; external 'ROCKET';
  procedure sx_CloseAll; far; external 'ROCKET';
  procedure sx_Commit; far; external 'ROCKET';
  procedure sx_Delete; far; external 'ROCKET';
  function  sx_Deleted:Integer; far; external 'ROCKET';
  function  sx_Eof:Integer; far; external 'ROCKET';
  function  sx_EvalNumeric(Expr:Pchar):Double; far; external 'ROCKET';
  function  sx_Found:Integer; far; external 'ROCKET';
  function  sx_GetDateJulian(FLDname:Pchar):Longint; far; external 'ROCKET';
  function  sx_GetDateString(FLDname:Pchar):Pchar; far; external 'ROCKET';
  function  sx_GetDouble(FLDname:Pchar):Double; far; external 'ROCKET';
  function  sx_GetInteger(FLDname:Pchar):Integer; far; external 'ROCKET';
  function  sx_GetLogical(FLDname:Pchar):Integer; far; external 'ROCKET';
  function  sx_GetLong(FLDname:Pchar):Longint; far; external 'ROCKET';
  function  sx_GetMemo(FLDname:Pchar;LineWidth:Integer):Pchar;
    far; external 'ROCKET';
  function  sx_GetString(FLDname:Pchar):Pchar; far; external 'ROCKET';
  function  sx_GetTrimString(FLDname:Pchar):Pchar; far; external 'ROCKET';
  procedure sx_Go(ToRec:LongInt); far; external 'ROCKET';
  procedure sx_GoBottom; far; external 'ROCKET';
  procedure sx_GoTop; far; external 'ROCKET';
  function  sx_IndexOrd:Integer; far; external 'ROCKET';
  function  sx_IndexKey:Pchar; far; external 'ROCKET';
  function  sx_Locked(RecNo:LongInt):Integer; far; external 'ROCKET';
  procedure sx_Pack; far; external 'ROCKET';
  procedure sx_Recall; far; external 'ROCKET';
  function  sx_RecCount:Longint; far; external 'ROCKET';
  function  sx_RecNo:Longint; far; external 'ROCKET';
  procedure sx_Replace(FLDname:Pchar;FldType:Integer;PtrData:Pchar);
    far; external 'ROCKET';
  function  sx_Rlock(RecNo:Longint):Integer; far; external 'ROCKET';
  function  sx_Seek(aPattern:Pchar):Integer; far; external 'ROCKET';
  function  sx_Select(AreaNo:Integer):Integer; far; external 'ROCKET';
  procedure sx_SetDeleted(OnOff:Integer); far; external 'ROCKET';
  procedure sx_SetExact(OnOff:Integer); far; external 'ROCKET';
  function  sx_SetHandles(ToCnt:Integer):Integer; far; external 'ROCKET';
  function  sx_SetOrder(ToIndexNo:Integer):Integer; far; external 'ROCKET';
  procedure sx_SetStringType(cstyle:Integer); far; external 'ROCKET';
  procedure sx_Skip(MoveCnt:Longint); far; external 'ROCKET';
  function  sx_TagArea(TagName:Pchar):Integer; far; external 'ROCKET';
  function  sx_TagName(TagIndex:Integer):Pchar; far; external 'ROCKET';
  procedure sx_Unlock(RecNo:Longint); far; external 'ROCKET';
  function  sx_Use(Fname:pchar;dAlias:pchar;OpenMode:Integer;
    RDDtype:Integer): Integer;
    far; external 'ROCKET';
  function  sx_WorkArea(AliasName:Pchar):Integer; far; external 'ROCKET';
  procedure sx_Zap; far; external 'ROCKET';

  { True/False tester for DLL boolean (integer) return values }
  function tf(AnInt:Integer):Boolean;
  begin
    Result:=Not (AnInt=0);
  end;

  { Get file name (for aliases), no path, no extension }
  function CoreFile(FullPath:string):string;
  var ii:integer;
  begin
    ii:=pos('\',FullPath);
    while ii>0 do
    begin
      FullPath:=Copy(FullPath,ii+1,100);
      ii:=pos('\',FullPath);
    end;
    ii:=pos('.',FullPath);
    if ii>1 then FullPath:=Copy(FullPath,1,ii-1);
    Result:=FullPath;
  end;

  { delay for interval }
  procedure delay(ForSeconds:integer);
  var tt:TDateTime;
      hr,thr,mn,sc,ms:word;
      ll,cur,rng:LongInt;
  begin
    tt:=now;
    rng:=ForSeconds;
    DecodeTime(tt,hr,mn,sc,ms);
    thr:=hr;
    cur:=(hr*3600)+(mn*60)+sc;
    ll:=(hr*3600)+(mn*60)+sc;
    while rng>(ll-cur) do
    begin
      tt:=now;
      DecodeTime(tt,hr,mn,sc,ms);
      if hr<thr then hr:=hr+24;
      ll:=(hr*3600)+(mn*60)+sc;
    end;
  end;

  function DBServer.s(fldname:string): string;
  begin
    Result:=StrPas(sx_GetString(StrPCopy(StrNull1,fldname)));
  end;

  function DBServer.st(fldname:string): string;
  begin
    Result:=StrPas(sx_GetTrimString(StrPCopy(StrNull1,fldname)));
  end;

  function DBServer.l(fldname:string): longint;
  begin
    Result:=sx_GetLong(StrPCopy(StrNull1,fldname));
  end;

  function DBServer.i(fldname:string): integer;
  begin
    Result:=sx_GetInteger(StrPCopy(StrNull1,fldname));
  end;

  function DBServer.b(fldname:string): boolean;
  begin
    Result:=tf(sx_GetLogical(StrPCopy(StrNull1,fldname)));
  end;

  function DBServer.f(fldname:string): double;
  begin
    { Minor bug: can't use sx_GetDouble yet }
    { S/B Result:=sx_GetDouble(StrPCopy(StrNull1,fldname)); }
    Result:=StrToFloat(StrPas(sx_GetString(StrPCopy(StrNull1,fldname))));
  end;

  function DBServer.m(fldname:string):pchar;
  begin
  end;

  function DBServer.d(fldname:string): string;  { date of form 04/15/95 }
  begin
    Result:=StrPas(sx_GetDateString(StrPCopy(StrNull1,fldname)));
  end;

  function DBServer.dj(fldname:string):longint; { date as Julian date}
  begin
    Result:=sx_GetDateJulian(StrPCopy(StrNull1,fldname));
  end;

  procedure DBServer.sa(fldname:string;newval:string);
  begin
    sx_Replace(StrPCopy(StrNull1,fldname),r_char,StrPCopy(StrNull2,newval));
  end;

  procedure DBServer.la(fldname:string;newval:longint);
  begin
    sx_Replace(StrPCopy(StrNull1,fldname),r_long,@newval);
  end;

  procedure DBServer.ia(fldname:string;newval:integer);
  begin
    sx_Replace(StrPCopy(StrNull1,fldname),r_integer,@newval);
  end;

  procedure DBServer.ba(fldname:string;newval:boolean);
  var ii:integer;
  begin
    if newval then ii:=1
              else ii:=0;
    sx_Replace(StrPCopy(StrNull1,fldname),r_logical,@ii);
  end;

  procedure DBServer.fa(fldname:string;newval:double);
  begin
    sx_Replace(StrPCopy(StrNull1,fldname),r_double,@newval);
  end;

  procedure DBServer.ma(fldname:string;newval:pchar);
  begin
  end;

  procedure DBServer.da(fldname:string;newval:string);
  begin
    sx_Replace(StrPCopy(StrNull1,fldname),r_datestr,StrPCopy(StrNull2,newval));
  end;

  function DBServer.Alias:string;
  begin
    Result:=StrPas(sx_Alias(0));
  end;

  procedure DBServer.Append;
  begin
    sx_AppendBlank;
  end;

  procedure DBServer.Bottom;
  begin
    sx_GoBottom;
  end;

  procedure DBServer.Close;
  begin
    sx_Close;
  end;

  procedure DBServer.CloseAll;
  begin
    sx_CloseAll;
  end;

  procedure DBServer.Delete;  { mark record as deleted }
  begin
    sx_Delete;
  end;

  function DBServer.Deleted: boolean; { status of deletion flag for record }
  begin
    Result:=tf(sx_Deleted);
  end;

  function DBServer.Eof: boolean;
  begin
    result:=tf(sx_Eof);
  end;

  procedure DBServer.Go(recnum:longint);
  begin
    sx_Go(recnum);
  end;

  function DBServer.LastRec: longint;
  begin
    Result:=sx_Reccount;
  end;

  function DBServer.Lock: boolean;   { try lock until succeeds }
  var ii:integer;
      res:boolean;
  begin
    ii:=0;
    res:=False;
    while (ii<10) and (not res) do
    begin
      res:=tf(sx_rLock(sx_Recno));
      if not res then delay(5);
      ii:=ii+1;
    end;
    Result:=res;
  end;

  function DBServer.aLock: boolean;  { try a few times then return }
  begin
  end;

  procedure DBServer.LoadTags;
  var ii,CurIndex:integer;
  begin
    tagcnt:=0;
    CurIndex:=sx_IndexOrd;
    for ii:=1 to MaxTags do
    begin
      tags[ii]:=StrPas(sx_TagName(ii));
      if length(tags[ii])>0 then
      begin
        tagcnt:=tagcnt+1;
        sx_SetOrder(ii);
        keys[ii]:=StrPas(sx_IndexKey);
      end else break;
    end;
    sx_SetOrder(CurIndex);
  end;

  procedure DBServer.Pack;
  begin
    sx_Pack;
  end;

  function DBServer.Recall: boolean;  { unmark record as deleted }
  begin
    sx_Recall;
  end;

  function DBServer.RecCount: longint;
  begin
    Result:=sx_RecCount;
  end;

  function DBServer.RecNo: longint;
  begin
    Result:=sx_RecNo;
  end;

  function DBServer.Seek(apattern:string): boolean;
  begin
    Result:=tf(sx_Seek(StrPCopy(StrNull1,apattern)));
  end;

  procedure DBServer.Select(AliasName:string);
  begin
    sx_Select(sx_WorkArea(StrPCopy(StrNull1,AliasName)));
  end;

  procedure DBServer.SetOrder(ToIndex:integer);
  begin
    sx_SetOrder(ToIndex);
  end;

  procedure DBServer.Skip;
  begin
    sx_Skip(1);
  end;

  procedure DBServer.Skip2(ByCnt:integer);
  begin
    sx_Skip(ByCnt);
  end;

  procedure DBServer.TagOrder(OrderByTag:String);
  begin
    sx_SetOrder(sx_TagArea(StrPCopy(StrNull1,OrderByTag)));
  end;

  procedure DBServer.Top;
  begin
    sx_GoTop;
  end;

  procedure DBServer.unLock;
  begin
    sx_Commit;
    sx_unLock(sx_Recno);
  end;

  function DBServer.Use(dbfname:string): boolean;
  var aliasname:string;
      ii:integer;
  begin
    aliasname:=CoreFile(dbfname);
    ii:=sx_Use(StrPCopy(StrNull1,dbfname),
      StrPCopy(StrNull2,aliasname),sx_READWRITE,sx_DBFCDX);
    if ii>0 then
    begin
      sx_SetOrder(1);
      Result:=True;
    end else Result:=False;
  end;

  function DBServer.UseWithAlias(dbfname:string;AliasName:string): boolean;
  var ii:integer;
  begin
    ii:=sx_Use(StrPCopy(StrNull1,dbfname),
      StrPCopy(StrNull2,aliasname),sx_READWRITE,sx_DBFCDX);
    if ii>0 then
    begin
      sx_SetOrder(1);
      Result:=True;
    end else Result:=False;
  end;

  function DBServer.UseAliasExcl(dbfname:string;AliasName:string): boolean;
  var ii:integer;
  begin
    ii:=sx_Use(StrPCopy(StrNull1,dbfname),
      StrPCopy(StrNull2,aliasname),sx_EXCLUSIVE,sx_DBFCDX);
    if ii>0 then
    begin
      sx_SetOrder(1);
      Result:=True;
    end else Result:=False;
  end;

  function DBServer.UseExclusive(dbfname:string): boolean;
  var aliasname:string;
      ii:integer;
  begin
    aliasname:=CoreFile(dbfname);
    ii:=sx_Use(StrPCopy(StrNull1,dbfname),
      StrPCopy(StrNull2,aliasname),sx_EXCLUSIVE,sx_DBFCDX);
    if ii>0 then
    begin
      sx_SetOrder(1);
      Result:=True;
    end else Result:=False;
  end;

  procedure DBServer.Zap;
  begin
    sx_Zap;
  end;

initialization

  StrNull1:=StrAlloc(80);
  StrNull2:=StrAlloc(80);
  sx_SetStringType(1);
  sx_SetHandles(40);
  sx_SetDeleted(0);

end.

