{ Created : 1993-12-01

Simple log file based Post Mortem Debugger

Install by calling InstallPMD *after* calling BBError.InstallExitHandler

InitIntHandler is not stable yet!

Last changes :
94-09-30  Added Windows GPF handler
94-10-03  Added procedure DonePMD
94-10-07  Improved stack walking a bit to detect near calls better
}



{$IFDEF MsDos}
{$F+,O+}
{$ENDIF}
{$IFDEF DPMI}
{$S-}
{$ENDIF}
{$IFDEF Windows}
{$S+}
{$ENDIF}

{$X+}

unit PMD;

interface


{ flags to pass to InitPMD to set PMD capabilities }
const
  dfStandard = 0;       { always make a symbolic stack dump }
  dfDataSeg = 1;        { optionally dump the data segment }


procedure InitPMD(AOptions : word);
{$IFDEF Windows}
procedure InitIntHandler;
{$ENDIF}
procedure DonePMD;



implementation

uses
  Objects,
  {$IFDEF Windows}
  Strings,
  WinAPI, WinTypes, WinProcs,
  ToolHelp,
  {$ENDIF}
  {$IFDEF DPMI}
  WinAPI,
  {$ENDIF}
  BBError, BBFile, BBUtil,
  TDInfo;



var
  Options : word;       { PMD flags }
  SaveDumpStack : DumpStackProcedureType;



{****************************************************************************}
{* DumpProcedure                                                            *}
{****************************************************************************}

procedure DumpProcedure(Addr : pointer; StackFrame : word);
{ PRE - Addr is a logical address pointer
 POST -
}
var
  First : Boolean;

  procedure WriteParameter(Symbol : PSymbol);  far;
  begin
    if First
     then  begin
       write(ferr, '(');
       First := FALSE;
     end
     else  write(ferr, ',');
    with Symbol^ do  begin
      if TypeIndex <> tid_Void then
        write(ferr, ItsValueStr(StackFrame));
    end; { of with }
  end;

var
  LineNumber : PLineNumber;
  Symbol : PSymbol;
  s : string;
begin
  New(LineNumber, AtAddr(Addr));
  if LineNumber = nil
   then  begin
     writeln(ferr, '  ', HexStr(PtrRec(Addr).Seg), ':', HexStr(PtrRec(Addr).Ofs));
   end
   else  begin
     write(ferr, '  ', LineNumber^.ItsCorrelation^.ItsSourceFile^.ItsName, ' (', LineNumber^.Value, ') ');

   {* display procedure where error occured *}
     New(Symbol, AtSegment(LineNumber^.ItsCorrelation^.ItsSegment, Addr));
     if Symbol <> nil then  begin
       if Symbol^.ItsType^.ReturnType = 1
        then  write(ferr, 'procedure ')
        else  write(ferr, 'function ');
       if Symbol^.ItsType^.ID = tid_SpecialFunc then  begin
         write(ferr, Symbol^.ItsType^.ItsClassType^.ItsName, '.');
       end;
       s := Symbol^.ItsName;
       write(ferr, s);
       First := TRUE;
       Symbol^.ItsScope^.ForEachParameter(@WriteParameter);
       if not First then  write(ferr, ')');
       write(ferr, ';');
       Dispose(Symbol, Done);
     end;
     writeln(ferr);
     Dispose(LineNumber, Done);
  end;
end;



{****************************************************************************}
{* FullDumpStack                                                            *}
{****************************************************************************}

procedure FullDumpStack(Addr : pointer; StackFrame : word);  far;
{ PRE - Addr is a logical Addr
        StackFrame is valid and even
 POST - stack is dumped with full parameter info
}
var
  NextFrame : word;
  PrevAddr : pointer;   { contains previous logical addres }
  PrevSel : word;       { contains previous physical selector }

  function NearCall : Boolean;
  { checks if a certain call is near. Is most of the time right }
  var
    codeptr : pointer;
  begin
    NearCall := FALSE;
    if PrevAddr = nil then
      Exit;
    PtrRec(codeptr).Ofs := PtrRec(Addr).Ofs;
  {$IFDEF MsDos}
    PtrRec(codeptr).Seg := PrevSel;
  {$ELSE}
    if GetSelectorLimit(PrevSel) <= PtrRec(codeptr).Ofs then
      Exit;
    PtrRec(codeptr).Seg := AllocSelector(PrevSel);
    if PtrRec(codeptr).Seg = 0 then
      Exit;
  {$ENDIF}
    with PtrRec(codeptr) do
      NearCall := (MemW[SSeg:StackFrame+4] = NextFrame) or
                  ((Ofs >= 5) and (Mem[Seg:Ofs-3] = $E8) and (Mem[Seg:Ofs-5] <> $9A));
  {$IFNDEF MsDos}
    FreeSelector(PtrRec(codeptr).Seg);
  {$ENDIF}
  end;

begin
  if not TDInfoPresent(nil) then  begin
    SaveDumpStack(Addr, StackFrame);
    Exit;
  end;
  LogError('*** Full stack dump ***');

  if Odd(StackFrame) then  Dec(StackFrame);
  NextFrame := MemW[SSeg:StackFrame];
  if Odd(NextFrame) then  Dec(NextFrame);
  PrevAddr := Addr;
  asm
    mov  ax,cs
    mov  PrevSel,ax
  end;

  while (NextFrame > StackFrame) and (MemW[SSeg:StackFrame] <> 0) do  begin
    PtrRec(Addr).Ofs := MemW[SSeg:StackFrame+2];
    if NearCall
     then  PtrRec(Addr).Seg := PtrRec(PrevAddr).Seg
     else  begin
       PrevSel := MemW[SSeg:StackFrame+4];
       PtrRec(Addr).Seg := PrevSel;
       Addr := GetLogicalAddr(Addr);
       if Addr = nil then  break;
     {$IFDEF MSDOS}
       {if (MemW[SSeg:NextFrame] = 0) or
          (MemW[SSeg:NextFrame] = MemW[SSeg:StackFrame+4]) then
         PtrRec(Addr).Seg := 0;}
     {$ELSE}
       if PtrRec(Addr).Seg = 0 then
         PtrRec(Addr).Seg := PtrRec(PrevAddr).Seg;
     {$ENDIF}
     end;
    StackFrame := NextFrame;
    DumpProcedure(Addr, StackFrame);
    NextFrame := MemW[SSeg:StackFrame];
    if Odd(NextFrame) then  Dec(NextFrame);
    PrevAddr := Addr;
  end; { of while }
  Flush(ferr);
end;


{****************************************************************************}
{* DumpDSeg                                                                 *}
{****************************************************************************}

procedure DumpDSeg(StackFrame : word);
var
  Module : PModule;

  procedure Dump(Symbol : PSymbol);  far;
  begin
    write(ferr, Module^.ItsName, '.', Symbol^.ItsName, ' : ');
    if Symbol^.ItsType = nil
     then  write(ferr, '<no type info>')
     else  write(ferr, Symbol^.ItsType^.ItsName);
    writeln(ferr, ' = ', Symbol^.ItsValueStr(StackFrame), ';');
  end;

var
  i : integer;
begin
  for i := 1 to DebugHeader.ModulesCount do  begin
    Module := New(PModule, Init(i));
    if Module <> nil then
      Module^.ForEachDSegElement(@Dump);
    Discard(Module);
  end; { of for }
end;


{****************************************************************************}
{* PostMortemDump                                                           *}
{****************************************************************************}

procedure PostMortemDump(StackFrame : word);  far;
var
  S : PStream;
  i : word;
  Segment : PSegment;
{$IFDEF Windows}
  Dest : array[0..127] of char;
{$ENDIF}
begin
{$IFDEF Windows}
  S := New(PBufStream, Init(StrPCopy(Dest, ParamStr(0)), stOpen+fmDenyNone, 512));
{$ELSE}
  S := New(PBufStream, Init(ParamStr(0), stOpen+fmDenyNone, 512));
{$ENDIF}
  if (S = nil) or (S^.Status <> stOK) then  begin
    if S = nil
     then  LogError('PMD: Stream allocation returned nil.')
     else  LogError('PMD: Error when opening stream. Status = ' + StrI(S^.Status));
    Exit;
  end;
  if not TDInfoPresent(S) then  begin
    LogError('PMD: Debug info not present.');
    LogError('Error ' + StrW(ExitCode) + ' at ' + HexStr(PtrRec(ErrorAddr).Seg) + ':' + HexStr(PtrRec(ErrorAddr).Ofs));
    SaveDumpStack(nil, StackFrame);
    Dispose(S, Done);
    Exit;
  end;

{* log errornumber and erroraddress *}
  LogError('Error ' + StrW(ExitCode) + ' at ' + HexStr(PtrRec(ErrorAddr).Seg) + ':' + HexStr(PtrRec(ErrorAddr).Ofs));
  LogError('MemAvail: ' + StrL(MemAvail));

{* dump the stack *}
  StackFrame := MemW[SSeg:StackFrame];
  if Odd(StackFrame) then  Dec(StackFrame);
  write(ferr, GetDateStr, ' ', GetTimeStr);
  DumpProcedure(ErrorAddr, StackFrame);
  FullDumpStack(ErrorAddr, StackFrame);

{* optionally dump Data segment *}
  if Options and dfDataSeg <> 0 then
    DumpDSeg(StackFrame);

  Dispose(Names, Done);

  Dispose(S, Done);

  ErrorAddr := nil;
end;


{$IFDEF Windows}
{ needs the pmdwin.dll library }
type
  IntHandlerProcedure = procedure(Int : word; StackFrame : word; wSS, wCS, wIP, wBP : word);

procedure InstallPMDWin(Task : THandle; Proc : IntHandlerProcedure);  far;  external 'pmdwin' index 1;
procedure UninstallPMDWin;  far;  external 'pmdwin' index 2;


procedure IntHandler(Int : word; StackFrame : word; wSS, wCS, wIP, wBP : word);  export;
var
  Addr : pointer;
  i : integer;
begin
  LogError('Fault: 0' + HexStr(Int) + 'h');
  if not TDInfoPresent(nil) then
    Exit;
  StackFrame := MemW[SSeg:StackFrame];
  if Odd(StackFrame) then
    Dec(StackFrame);
  Addr := Ptr(wCS, wIP);
  DumpProcedure(GetLogicalAddr(Addr), StackFrame);
  DumpStack(GetLogicalAddr(Addr), StackFrame);
  Close(ferr);
  UninstallPMDWin;
  TerminateApp(0, No_UAE_Box);
end;

procedure InitIntHandler;
begin
  InstallPMDWin(GetCurrentTask, IntHandler);
end;


function WindowsPostMortemDebugger(wID : word; dwData : longint): Bool ;  export;
{ called on error conditions }
var
  NFYLogError : PNFYLogError absolute dwData;
  NFYLogParamError : PNFYLogParamError absolute dwData;
  StackFrame : word;
begin
  case wID of
    nfy_RIP : LogError('RIP Error');
    nfy_OutStr : LogError(StrPas(PChar(dwData)));
    nfy_LogError : LogError('Windows log error: ' + StrW(NFYLogError^.wErrCode));
    nfy_LogParamError : begin
        LogError('Windows parameter error: ' + StrW(NFYLogParamError^.wErrCode));
        asm
          mov StackFrame,bp
        end;
        DumpStack(GetLogicalAddr(NFYLogParamError^.lpfnErrorAddr), StackFrame);
      end;
  end; { of case }

{ always return FALSE, so other notification procs see this one also }
  WindowsPostMortemDebugger := FALSE;

end;
{$ENDIF}


procedure InitPMD(AOptions : word);
begin
  if IsFileOpen(ferr) then  begin
    Options := AOptions;
    HandleRunTimeError := PostMortemDump;
    SaveDumpStack := DumpStack;
    DumpStack := FullDumpStack;
  {$IFDEF Windows}
    NotifyRegister(0, WindowsPostMortemDebugger, nf_RIP);
  {$ENDIF}
    LogError('Post Mortem Debugger installed.');
  end;
end;


procedure DonePMD;
{ undoes memory allocations made if PMD was activated }
begin
  Discard(Names);
  Discard(DStream);
end;


end.  { of unit PMD }
